VARCHAR = "varchar"
CHAR = "char"
CHARACTER = "character"
STRING = "string"  # Synonymous with VARCHAR
TEXT = "text"  # Synonymous with VARCHAR

BINARY = "binary"  # No notion of unicode characters, so length always measured in terms of bytes
VARBINARY = "varbinary"  # Synonymous with BINARY
