class InvalidParamsException(Exception):
    pass


class InvalidWhitespaceException(Exception):
    pass


class UnsupportedFeatureException(Exception):
    pass


class InvalidColumnConstraintException(Exception):
    pass
